@@ -548,7 +548,11 @@ static char *(features[]) =
 	"-smartindent",
 #endif
 #ifdef FEAT_SODIUM
+# ifdef DYNAMIC_SODIUM
+	"+sodium/dyn",
+# else
 	"+sodium",
+# endif
 #else
 	"-sodium",
 #endif
@@ -746,6 +750,116 @@ static char *(features[]) =
 
 static int included_patches[] =
 {   /* Add new patch number below this line */
+/**/
+    4383,
+/**/
+    4382,
+/**/
+    4381,
+/**/
+    4380,
+/**/
+    4379,
+/**/
+    4378,
+/**/
+    4377,
+/**/
+    4376,
+/**/
+    4375,
+/**/
+    4374,
+/**/
+    4373,
+/**/
+    4372,
+/**/
+    4371,
+/**/
+    4370,
+/**/
+    4369,
+/**/
+    4368,
+/**/
+    4367,
+/**/
+    4366,
+/**/
+    4365,
+/**/
+    4364,
+/**/
+    4363,
+/**/
+    4362,
+/**/
+    4361,
+/**/
+    4360,
+/**/
+    4359,
+/**/
+    4358,
+/**/
+    4357,
+/**/
+    4356,
+/**/
+    4355,
+/**/
+    4354,
+/**/
+    4353,
+/**/
+    4352,
+/**/
+    4351,
+/**/
+    4350,
+/**/
+    4349,
+/**/
+    4348,
+/**/
+    4347,
+/**/
+    4346,
+/**/
+    4345,
+/**/
+    4344,
+/**/
+    4343,
+/**/
+    4342,
+/**/
+    4341,
+/**/
+    4340,
+/**/
+    4339,
+/**/
+    4338,
+/**/
+    4337,
+/**/
+    4336,
+/**/
+    4335,
+/**/
+    4334,
+/**/
+    4333,
+/**/
+    4332,
+/**/
+    4331,
+/**/
+    4330,
+/**/
+    4329,
 /**/
     4328,
 /**/
